###
###  AUTHOR:  Arnost Komarek
###
###  LOG:     20130814  created
###                     
###
### ====================================

LSest <- function(x, L, alternative = c("two.sided", "less", "greater"), conf.level = 0.95, theta0 = 0)
{
  if (class(x) != "lm") stop("x must be an object of class lm.")
  if (conf.level <= 0 | conf.level >= 1) stop("conf.level must be between 0 and 1.")

  if (!is.numeric(L)) stop("L must be numeric")
  if (!is.matrix(L)){
    L <- matrix(L, nrow = 1)
    colnames(L) <- names(L)
  }

  if (ncol(L) != length(coef(x))) stop("L must be either a vector of length ", length(coef(x)), " or a matrix with ", length(coef(x)), " columns.", sep = "")     

  if (length(theta0) == 1) theta0 <- rep(theta0, nrow(L))
  if (length(theta0) != nrow(L)) stop("theta0 must be either a number or a vector of length ", nrow(L), ".", sep = "")
  
  theta     <- as.numeric(L %*% coef(x))
  var.theta <- L %*% vcov(x) %*% t(L)
  se.theta  <- sqrt(diag(var.theta))
  Z.theta   <- (theta - theta0) / se.theta

  alternative <- match.arg(alternative)
  switch (alternative,
    "two.sided" = {
      qq <- qt(1 - (1 - conf.level)/2, df = x$df.residual)
      low.theta <- theta - qq * se.theta
      upp.theta <- theta + qq * se.theta
      pv.theta  <- 2 * pt(-abs(Z.theta), df = x$df.residual)
    },
    "less" = {
      qq <- qt(conf.level, df = x$df.residual)
      low.theta <- rep(-Inf, length(theta))
      upp.theta <- theta + qq * se.theta
      pv.theta  <- pt(Z.theta, df = x$df.residual)      
    },
    "greater" = {
      qq <- qt(conf.level, df = x$df.residual)
      low.theta <- theta - qq * se.theta
      upp.theta <- rep(Inf, length(theta))
      pv.theta  <- pt(Z.theta, df = x$df.residual, lower.tail = FALSE)      
    }  
  )
  
  RET <- data.frame(theta, se.theta, Z.theta, pv.theta, low.theta, upp.theta)
  colnames(RET) <- c("Estimate", "Std. Error", "t value", "P value", "Lower", "Upper")
  rownames(RET) <- rownames(L)

  class(RET) <- c("LSest")
  return(RET)  
}  


print.LSest <- function(x, ...)
{
  x[["P value"]] <- format.pval(x[["P value"]])
  cnx <- names(x)
  rnx <- attr(x, "row.names")
  x <- data.frame(x[["Estimate"]],
                  x[["Std. Error"]],
                  x[["t value"]],
                  x[["P value"]],
                  x[["Lower"]],
                  x[["Upper"]])
  colnames(x) <- cnx
  rownames(x) <- rnx
  
  print(x, ...)
  return(invisible(x))  
}  
